/***************************************************************************
Copyright (c) 2006 Peter Korzuszek, Damian Rakowski, KateOS Project
 
Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:
 
*Redistributions of source code must retain the above copyright notice,
this list of conditions and the following disclaimer.
 
*Redistributions in binary form must reproduce the above copyright notice,
this list of conditions and the following disclaimer in the documentation
and/or other materials provided with the distribution.
 
THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS ``AS
IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE
LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
INTERRUPTION)
HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
DAMAGE. 
****************************************************************************/

#ifndef SMART_TOOLS_THREAD_H
#define SMART_TOOLS_THREAD_H

#include <pthread.h>

#include "smarttools.h"

/*!
	\brief This class is used to create seperate execution thread.
	
	To create thread, just implement this class in yours and write down
	your own run() function. This will be code executed in your thread.
	
	To run thread use start() function.
*/

class StThread
{
	public:
		StThread();
		virtual ~StThread(){};
		
		/*! Run the thread. */
		void start();
		/*! Terminate the thread immediately. */
		void terminate();
		
		/*! \return true if thread has finished its task. */
		bool hasFinished() const { return finished; }
		
	protected:
		/*! You must implement this function to make StThread work. Inside should be code that will be executed in thread. */
		virtual void run() = 0;
	
	private:
		pthread_t thread;
		volatile bool finished;
		
		void init(){ finished = true; }
		static void* startThread( void* );
};

#endif //SMART_TOOLS_THREAD_H
