/***************************************************************************
Copyright (c) 2006 Peter Korzuszek, Damian Rakowski, KateOS Project
 
Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:
 
*Redistributions of source code must retain the above copyright notice,
this list of conditions and the following disclaimer.
 
*Redistributions in binary form must reproduce the above copyright notice,
this list of conditions and the following disclaimer in the documentation
and/or other materials provided with the distribution.
 
THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS ``AS
IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE
LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
INTERRUPTION)
HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
DAMAGE. 
****************************************************************************/

#ifndef SMART_TOOLS_TIME_H
#define SMART_TOOLS_TIME_H

#include <time.h>
#include <unistd.h>

#define ST_SLEEP( msec ) \
	sleep( msec/1000 ); \
	usleep( (msec-((msec/1000)*1000))*1000 )

/*!
   \brief Manage time and date
   
   It allows you to getCurrentTime, setTime, setDate,
   check which time is earlier, greater or equal.
*/
class StTime
{
	public:
		StTime(){ init(); }
		StTime(time_t time);
		void setTime( int year, int month, int day, int hour, int minute, int second ){ this->year = year; this->month = month; this->day = day; this->hour = hour; this->min = minute; this->sec = second; }
		void setTime( time_t time );
		void setTime( tm * time );
		
		/**Get the time stored by class.*/
		time_t getTime();
		
		/**Set the year.*/
		void setYear(int year){ this->year = year; }
		/**Set the month.*/
		void setMonth(int month){ this->month = month; }
		/**Set the day.*/
		void setDay(int day){ this->day = day; }
		/**Set the hour.*/
		void setHour(int hour){ this->hour = hour; }
		/**Set the minute.*/
		void setMin(int minute){ this->min = minute; }
		/**Set the second.*/
		void setSec(int second){ this->sec = second; }
		
		/**Get the year.*/
		int getYear() const { return year; }
		/**Get the month.*/
		int getMonth() const { return month; }
		/**Get the day.*/
		int getDay() const { return day; }
		/**Get the hour.*/
		int getHour() const { return hour; }
		/**Get the minute.*/
		int getMin() const { return min; }
		/**Get the second.*/
		int getSec() const { return sec; }
		
		/*!
   			Compare the time stored by class with the time passed by argument.
   			
   			\param otherTime - time to compare
   			
   			\return True if time passed by argument is greater.
		*/
		bool greaterThan( const StTime& otherTime );
		/*!
   			Compare the time stored by class with the time passed by argument.
   			
   			\param otherTime - time to compare
   			
   			\return True if time passed by argument is lower.
		*/
		bool lowerThan( const StTime& otherTime );
		/*!
   			Compare the time stored by class with the time passed by argument.
   			
   			\param otherTime - time to compare
   			
   			\return True if time passed by argument is equal.
		*/
		bool equal( const StTime& otherTime );
		
		/** Same as greaterThan function */
		bool operator>( const StTime& otherTime ){ return greaterThan( otherTime ); }
		/** Same as lowerThan function */
		bool operator<( const StTime& otherTime ){ return lowerThan( otherTime ); }
		/** Same as equal function */
		bool operator==( const StTime& otherTime ){ return equal( otherTime ); }
		
		/** Get the current time*/
		static StTime getCurrentTime();
		/*!
   			Calculate the diffrence between the time passed by argument and the time stored by class.
   				
   			\return Diffrence in seconds. If the time passed by argument is greater the diffrence is
   			greater than 0, in other case in lower than 0.
		*/
		int diffrence( StTime time )
			{ return (int)difftime(time.getTime(), getTime()); }
		
		/**Set all variables to 0.*/
		void clear(){ year = month = day = hour = min = sec = 0; }

	private:
		int year;
		int month;
		int day;
		int hour;
		int min;
		int sec;
		
		void init(){ clear(); }
};

#endif //SMART_TOOLS_Time_H
